/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef BOUNDARYMESHITEMS_H
#define BOUNDARYMESHITEMS_H

#include "meshitem.h"


/**********************************************************
**
**  BOUNDARY MESH ITEM
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT BoundaryMeshItem : public MeshItem
{
  Q_OBJECT
public:
    BoundaryMeshItem();
    BoundaryMeshItem(const BoundaryMeshItem& other);
    ~BoundaryMeshItem();

    BoundaryMeshItem& operator=(const BoundaryMeshItem& other);
    virtual MeshItem* clone() const;

    virtual void setMesh(Mesh *mesh);

    virtual operator QVariant() const;

protected:
    void setNodeBoundary(MeshEngine& meshEngine,
                         MeshPoint point,
                         MeshEngine::MeshDirection direction);

};

Q_DECLARE_METATYPE(BoundaryMeshItem);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const BoundaryMeshItem& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, BoundaryMeshItem& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const BoundaryMeshItem& item);





/**********************************************************
**
**  LINEAR BOUNDARY MESH ITEM
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT LinearBoundary: public BoundaryMeshItem
{
  Q_OBJECT
public:
    LinearBoundary();
    LinearBoundary(const LinearBoundary& other);
    ~LinearBoundary();

    LinearBoundary& operator=(const LinearBoundary& other);
    virtual MeshItem* clone() const;
    virtual operator QVariant() const;

    virtual void setSize(const Size3D<float> size);
    virtual void addToMesh(MeshEngine& meshEngine);
    virtual void draw();
    virtual const QPixmap generateImage(const QSize &size,
                                        const QColor border,
                                        const QColor fill);
};

// STREAM OPERATIONS
Q_DECLARE_METATYPE(LinearBoundary);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const LinearBoundary& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, LinearBoundary& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const LinearBoundary& item);




/**********************************************************
**
**  RECTANGLE BOUNDARY MESH ITEM
**
**********************************************************/


class QTOCLMESHSHARED_EXPORT RectangleBoundary: public BoundaryMeshItem
{
  Q_OBJECT
private:
  LinearBoundary walls[4]; // [0]Top, [1]Left, [2]Bottom, [3]Right
public:
    RectangleBoundary();
    RectangleBoundary(const RectangleBoundary& other);
    ~RectangleBoundary();

    RectangleBoundary& operator=(const RectangleBoundary& other);
    virtual MeshItem* clone() const;
    virtual operator QVariant() const;

    virtual void setPos(const Point3D<float> pos);
    virtual void setUp(const Point3D<float> up);
    virtual void setSize(const Size3D<float> size);
    virtual void addToMesh(MeshEngine& meshEngine);
    virtual void setMesh(Mesh *mesh);
    virtual void draw();
    virtual const QPixmap generateImage(const QSize &size,
                                        const QColor border,
                                        const QColor fill);

};

// STREAM OPERATIONS
Q_DECLARE_METATYPE(RectangleBoundary);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const RectangleBoundary& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, RectangleBoundary& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const RectangleBoundary& item);


#endif // BOUNDARYMESHITEMS_H
