/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MATERIALMESHITEMS_H
#define MATERIALMESHITEMS_H

#include "meshitem.h"

/**********************************************************
**
**  MATERIAL MESH ITEM
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT MaterialMeshItem : public MeshItem
{
  Q_OBJECT
public:
    MaterialMeshItem();
    MaterialMeshItem(const MaterialMeshItem& other);
    ~MaterialMeshItem();

    MaterialMeshItem& operator=(const MaterialMeshItem& other);
    virtual MeshItem* clone() const;
    virtual operator QVariant() const;

protected:
    void setNodeProperty(MeshEngine& meshEngine,
                         MeshPoint point,
                         int propertyNum,
                         float value);
    void setEpsilonR(MeshEngine& meshEngine, const MeshPoint &pos);
    void setConductance(MeshEngine& meshEngine, const MeshPoint &pos);

};

Q_DECLARE_METATYPE(MaterialMeshItem);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const MaterialMeshItem& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, MaterialMeshItem& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const MaterialMeshItem& item);




/**********************************************************
**
**  RECTANGLE MATERIAL MESH ITEM
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT RectangularMaterial
    : public MaterialMeshItem
{
  Q_OBJECT
public:
    RectangularMaterial();
    RectangularMaterial(const RectangularMaterial& other);
    ~RectangularMaterial();

    RectangularMaterial& operator=(
        const RectangularMaterial& other);
    virtual MeshItem* clone() const;
    virtual operator QVariant() const;

    virtual void addToMesh(MeshEngine& meshEngine);
    virtual void draw();
    virtual const QPixmap generateImage(const QSize &size,
                                        const QColor border,
                                        const QColor fill);
};

// STREAM OPERATIONS
Q_DECLARE_METATYPE(RectangularMaterial);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const RectangularMaterial& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, RectangularMaterial& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const RectangularMaterial& item);









#endif // MATERIALMESHITEMS_H
