/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef SOURCEMESHITEMS_H
#define SOURCEMESHITEMS_H

#include "meshitem.h"

/**********************************************************
**
**  SOURCE MESH ITEM
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT SourceMeshItem : public MeshItem
{
  Q_OBJECT
private:
    QVector<float> data_;
public:
    SourceMeshItem();
    SourceMeshItem(const SourceMeshItem& other);
    ~SourceMeshItem();

    SourceMeshItem& operator=(const SourceMeshItem& other);
    virtual MeshItem* clone() const;
    virtual operator QVariant() const;

    virtual void draw();
    virtual void setSize(const Size3D<float> size);
    virtual void setMesh(Mesh *mesh);

//    virtual QVector<unsigned int> getSourceInformation() const;
    void clearData();
    const QVector<float>& getData() const;
    void setData(const QVector<float>& vector);
    void appendData(float* newData, size_t appendedElements);

    void addToMesh(MeshEngine &meshEngine);
    const QPixmap generateImage(const QSize &size,
                                const QColor border,
                                const QColor fill);


protected:
    virtual QVector<float> generateSourceData() const;
    virtual unsigned int getDirectionCode() const;
};

Q_DECLARE_METATYPE(SourceMeshItem);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const SourceMeshItem& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, SourceMeshItem& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const SourceMeshItem& item);




/**********************************************************
**
**  IMPULSE SOURCE MESH ITEM
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT ImpulseSource: public SourceMeshItem
{
  Q_OBJECT
public:
    ImpulseSource();
    ImpulseSource(const ImpulseSource& other);
    ~ImpulseSource();

    ImpulseSource& operator=(const ImpulseSource& other);
    virtual MeshItem* clone() const;
    virtual operator QVariant() const;
    virtual void addToMesh(MeshEngine &meshEngine);
    virtual const QPixmap generateImage(const QSize &size,
                                        const QColor border,
                                        const QColor fill);

protected:
    QVector<float> generateSourceData() const;

};

// STREAM OPERATIONS
Q_DECLARE_METATYPE(ImpulseSource);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const ImpulseSource& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, ImpulseSource& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const ImpulseSource& item);





/**********************************************************
**
**  SINUSOIDAL SOURCE MESH ITEM
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT SinusoidalSource: public SourceMeshItem
{
  Q_OBJECT
public:
    SinusoidalSource();
    SinusoidalSource(const SinusoidalSource& other);
    ~SinusoidalSource();

    SinusoidalSource& operator=(const SinusoidalSource& other);
    virtual MeshItem* clone() const;
    virtual operator QVariant() const;

//    virtual void addToMesh(MeshEngine& meshEngine);
    virtual const QPixmap generateImage(const QSize &size,
                                        const QColor border,
                                        const QColor fill);

protected:
    QVector<float> generateSourceData() const;

};

// STREAM OPERATIONS
Q_DECLARE_METATYPE(SinusoidalSource);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const SinusoidalSource& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, SinusoidalSource& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const SinusoidalSource& item);



















#endif // SOURCEMESHITEMS_H
