/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/


#ifndef TLMENGINE2D_H
#define TLMENGINE2D_H
#include "QtOclMesh_global.h"
#include "meshengine.h"

typedef unsigned short USHORT;
const USHORT NULL_BOUNDARY = 0;

class QTOCLMESHSHARED_EXPORT TLMEngine2D : public MeshEngine
{
  Q_OBJECT
public:
  TLMEngine2D(Mesh* initMesh);
  TLMEngine2D(const TLMEngine2D& other);
  ~TLMEngine2D();

  TLMEngine2D& operator=(const TLMEngine2D& other);
  virtual MeshEngine* clone() const;

  virtual void setNodeBoundary(const MeshPoint &point,
                               const MeshEngine::MeshDirection direction,
                               const float value);
  virtual void setNodePermeability(const MeshPoint &point,
                                   const QVector<float> permeability);
  virtual void setNodePermittity(const MeshPoint &point,
                                 const QVector<float> permittivity);
  virtual void setNodeConductance(const MeshPoint &point,
                                  const QVector<float> conductance);
  virtual void setNodeVoltage(const MeshPoint &point,
                              const int voltage, const float value);
  virtual float& getNodeVoltage(const MeshPoint &point, const int voltage);

  virtual void simulateIteration( int iteration );

  virtual void updateHost();

  virtual void createSimulation();


protected:
  virtual void allocateResources(const MeshSize &meshSize);
  virtual void initializeResources(const MeshSize &meshSize);
  virtual void releaseResources();

  virtual void scatterImpulses();
  virtual void impulseInterchange();
  virtual void exciteMesh(int iteration);
  virtual void sampleMesh(int iteration);

  const USHORT addBoundaryValue(const float value);
  const USHORT getBoundaryID(const float value);
  const int getMemoryIndex(const MeshPoint& point, int memberNumber) const;

private:
  void flipVoltages(MeshPoint point1, int voltage1, MeshPoint point2, int voltage2);

protected:
  float       boundaryTypeTable_[256];
  USHORT      numBoundaryTypes_;
  float*      nodeVoltages_;
  USHORT*     nodeBoundaries_;
  float*      nodeProperties_;


signals:

public slots:

};

#endif // TLMENGINE2D_H
