/*******************************************************************************
**  FILE:	    dielectric.cpp
**  DESCRIPTION:    Dielectric mesh class can represent a 2D Dielectric TLM mesh
**                  for simulating propagation through dielectric materials.
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "inc/dielectricmesh.h"
#include "inc/meshitems.h"
#include "inc/meshengine.h"
#include "inc/tlmengine2d.h"
#include "inc/tlmengine2dgpu.h"


DielectricMesh::DielectricMesh(QObject *parent) :
    Mesh(parent)
{
  // Setup supported boundary types
  addSupportedItem(new LinearBoundary);
  addSupportedItem(new RectangleBoundary);
  // Setup supported material types
  //supportedMaterialTypes_.append(new RectangleDielectricMaterial);
  // Setup supported source types
  addSupportedItem(new ImpulseSource);
  addSupportedItem(new SinusoidalSource);

  addSupportedItem(new ProbeMeshItem);

  addSupportedEngine(new TLMEngine2D(this));
  addSupportedEngine(new TLMEngine2DGPU(this));
}


DielectricMesh::~DielectricMesh()
{
}

