/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "inc/sourcemeshitems.h"
#include "inc/mesh.h"

/**********************************************************
**
**  SOURCE MESH ITEM
**
**********************************************************/


//-----------------------------------------------------------------------------
// CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


SourceMeshItem::SourceMeshItem():
	MeshItem()
{
  displayText_    = QString("Source");
  whatsThisText_  = QString("Source items are used for exciting the"
                            " mesh structure during simulation.");
  toolTipText_    = QString("Source Items");
  fillColor_      = QColor(0,0,255,128);
  borderColor_    = QColor(Qt::transparent);

//  addProperty(QString("direction y+"),QString("+Vy"),QVariant(true));
//  addProperty(QString("direction y-"),QString("-Vy"),QVariant(true));
//  addProperty(QString("direction x-"),QString("-Vx"),QVariant(true));
//  addProperty(QString("direction x+"),QString("+Vx"),QVariant(true));

  addProperty(QString("magnitude"),QString("Magnitude"),QVariant(1.0f));
  addProperty(QString("duration"),QString("Duration"),QVariant(250));
}



SourceMeshItem::SourceMeshItem(const SourceMeshItem &other):
	MeshItem  ( other ),
  data_     ( other.data_ )
{
}



SourceMeshItem::~SourceMeshItem()
{
}



//-----------------------------------------------------------------------------
// OPERATOR OVERLOADS
//-----------------------------------------------------------------------------


SourceMeshItem& SourceMeshItem::operator=(const SourceMeshItem& other)
{
  if(&other!=this){
    data_ = other.data_;
    MeshItem::operator =(other);
  }
  return *this;
}



MeshItem*
SourceMeshItem::clone() const
{
  return new SourceMeshItem(*this);
}



SourceMeshItem::operator QVariant() const
{
  return QVariant::fromValue(*this);
}


//-----------------------------------------------------------------------------
// PUBLIC METHODS
//-----------------------------------------------------------------------------


void SourceMeshItem::draw()
{
  QColor fill = getFillColor();
  GLUquadric * sphereQuad;
  GLdouble radius = size().width()/2.0;
  GLint slices, stacks;
  slices = 10;
  stacks = 10;

  Point3D<float> position = pos();

  sphereQuad = gluNewQuadric();

  glPushMatrix();

  glTranslatef(position.cx(),position.cy(),position.cz());
  //glTranslatef(-radius,-radius,radius);

  glColor3f(fill.redF(),fill.greenF(),fill.blueF());
  gluSphere(sphereQuad,radius,slices,stacks);


  glPopMatrix();

}


void SourceMeshItem::setSize(const Size3D<float> size)
{

}


void SourceMeshItem::setMesh(Mesh *mesh)
{
  MeshItem::setMesh(mesh);
  if(this->mesh() != 0 ){
    MeshItem::setSize(this->mesh()->getMeshResolution());
  }
}


const QPixmap
SourceMeshItem::generateImage(const QSize &size,
                              const QColor border,
                              const QColor fill)
{
  QPixmap pixmap = getPixmap(size);
  QPainter painter(&pixmap);
  setupPainter(painter,border,fill);

  painter.drawEllipse(QPoint(size.width()/2,size.height()/2),
                      size.width()/2,size.height()/2);
  painter.end();
  return pixmap;
}



void
SourceMeshItem::clearData()
{
  data_.clear();
}



const QVector<float>&
SourceMeshItem::getData() const
{
    return data_;
}



void
SourceMeshItem::setData(const QVector<float>& vector)
{
    data_ = vector;
}



void
SourceMeshItem::appendData(float *newData, size_t appendedElements)
{
  for(size_t i=0;i<appendedElements;++i){
    data_.append(newData[i]);
  }
}



void
SourceMeshItem::addToMesh(MeshEngine &meshEngine)
{
    setData(generateSourceData());
    meshEngine.addSource(this);
}



//-----------------------------------------------------------------------------
// PROTECTED METHODS
//-----------------------------------------------------------------------------


unsigned int
SourceMeshItem::getDirectionCode() const
{
    unsigned int tmpDirection = 0;
    tmpDirection += getPropertyValue("direction y+").toBool()*1;
    tmpDirection += getPropertyValue("direction x+").toBool()*2;
    tmpDirection += getPropertyValue("direction y-").toBool()*4;
    tmpDirection += getPropertyValue("direction x-").toBool()*8;
    return tmpDirection;
}



QVector<float>
SourceMeshItem::generateSourceData() const
{
  QVector<float> tmpVector;
  tmpVector.append(0.0f);
  return tmpVector;
}



//-----------------------------------------------------------------------------
// STREAM OPERATORS
//-----------------------------------------------------------------------------


QDebug
operator<<(QDebug dbg,const SourceMeshItem& item){
  dbg.nospace()<<"\nSource Mesh Item:"<< (MeshItem)item;
  dbg.nospace()<<"Source Data:"<<item.getData();
  return dbg.maybeSpace();
}



QDataStream&
operator<<(QDataStream &out, const SourceMeshItem& item){
  out << (MeshItem)item
      << item.getData();
  return out;
}



QDataStream&
operator>>(QDataStream &in, SourceMeshItem& item){
  MeshItem tmpItem;
  QVector<float> tmpVector;
  in  >> tmpItem
      >> tmpVector;
  (MeshItem)item = tmpItem;
  item.setData(tmpVector);
  return in;
}





/**********************************************************
**
**  IMPULSE DIELECTRIC SOURCE MESH ITEM
**
**********************************************************/


//-----------------------------------------------------------------------------
//  CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


ImpulseSource::ImpulseSource():
	SourceMeshItem()
{
    displayText_    = QString("Impulse");
    whatsThisText_  = QString("Mesh Item representing an impulse "
                             "excitation in the dielectric material.");
    toolTipText_    = QString("Impulse Dielectric Source Item");
    //SourceMeshItem::removeProperty(QString("duration"));
}



ImpulseSource::ImpulseSource(
    const ImpulseSource &other):
    SourceMeshItem(other)
{
}



ImpulseSource::~ImpulseSource()
{
}



//-----------------------------------------------------------------------------
//  OPERATOR OVERLOADS
//-----------------------------------------------------------------------------



ImpulseSource&
ImpulseSource::operator =(const ImpulseSource &other)
{
  if(&other!=this){
    SourceMeshItem::operator =(other);
  }
  return *this;
}



MeshItem*
ImpulseSource::clone() const
{
  return new ImpulseSource(*this);
}



ImpulseSource::operator QVariant() const
{
  return QVariant::fromValue(*this);
}


//-----------------------------------------------------------------------------
//  VIRTUAL METHOD DEFINITIONS
//-----------------------------------------------------------------------------


void
ImpulseSource::addToMesh(MeshEngine &meshEngine)
{
  setPropertyValue("SourceMeshItem/duration",1);
  SourceMeshItem::addToMesh(meshEngine);
}


const QPixmap
ImpulseSource::generateImage(const QSize &size,
                                       const QColor border,
                                       const QColor fill)
{
  return SourceMeshItem::generateImage(size,border,fill);
}



//-----------------------------------------------------------------------------
//  PROTECTED METHODS
//-----------------------------------------------------------------------------


QVector<float>
ImpulseSource::generateSourceData() const
{
  QVector<float> tmpVector;
  tmpVector.append(getPropertyValue("SourceMeshItem/magnitude").toFloat());
  return tmpVector;
}



//-----------------------------------------------------------------------------
//  STREAM OPERATORS
//-----------------------------------------------------------------------------


QDataStream&
operator<<(QDataStream &out, const ImpulseSource& item)
{
  out << (SourceMeshItem)item;
  return out;
}



QDataStream&
operator>>(QDataStream &in, ImpulseSource& item)
{
  SourceMeshItem tmpItem;
  in>>tmpItem;
  (SourceMeshItem)item = tmpItem;
  return in;
}



QDebug
operator<<(QDebug dbg,const ImpulseSource& item)
{
  dbg.nospace()<<"\nImpulse Dielectric Source Item:"
      << (SourceMeshItem)item;
  return dbg.maybeSpace();
}




/**********************************************************
**
**  SINUSOIDAL DIELECTRIC SOURCE MESH ITEM
**
**********************************************************/


//-----------------------------------------------------------------------------
//  CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


SinusoidalSource::SinusoidalSource():
	SourceMeshItem()
{
    displayText_    = QString("Sinusoidal Source");
    whatsThisText_  = QString("Mesh Item representing a sinusoidal "
                             "excitation in the dielectric material.");
    toolTipText_    = QString("Sinusoidal Dielectric Source Item");
    fillColor_      = QColor(128,128,0,128);
    borderColor_    = QColor(Qt::transparent);

    addProperty(QString("frequency"),QString("Frequency"),QVariant(15));
}



SinusoidalSource::SinusoidalSource(
    const SinusoidalSource &other):
    SourceMeshItem(other)
{

}



SinusoidalSource::~SinusoidalSource()
{
}



//-----------------------------------------------------------------------------
//  OPERATOR OVERLOADS
//-----------------------------------------------------------------------------


SinusoidalSource&
SinusoidalSource::operator=(const SinusoidalSource &other)
{
  if(&other!=this){
    SourceMeshItem::operator =(other);
  }
  return *this;
}



MeshItem*
SinusoidalSource::clone() const
{
  return new SinusoidalSource(*this);
}



SinusoidalSource::operator QVariant() const
{
  return QVariant::fromValue(*this);
}




//-----------------------------------------------------------------------------
//  VIRTUAL METHOD DEFINITIONS
//-----------------------------------------------------------------------------


//void
//SinusoidalSource::addToMesh(MeshEngine &meshEngine)
//{

//}



const QPixmap SinusoidalSource::generateImage(const QSize &size,
                                                        const QColor border,
                                                        const QColor fill)
{
  return SourceMeshItem::generateImage(size,border,fill);
}



//-----------------------------------------------------------------------------
//  PROTECTED METHODS
//-----------------------------------------------------------------------------



QVector<float> SinusoidalSource::generateSourceData() const
{
  QVector<float> tmpVector;
  const float PI          = 3.14159265;
  const float lightSpeed  = 299792458;
  float deltaL = mesh()->getMeshResolution().width();

  float mag = getPropertyValue("SourceMeshItem/magnitude").toFloat();
  size_t duration = getPropertyValue("SourceMeshItem/duration").toUInt();
  float frequency = getPropertyValue_("frequency").toFloat();
  frequency *= 1E9;
  float timeStep = deltaL/(2*lightSpeed);
  float value = 0;
  for(size_t i=0;i<duration;++i){
    value = mag*sin(2*PI*frequency*i*timeStep);
    tmpVector.append(value);
  }
  return tmpVector;
}



//-----------------------------------------------------------------------------
//  STREAM OPERATORS
//-----------------------------------------------------------------------------


QDataStream&
operator<<(QDataStream &out, const SinusoidalSource& item)
{
  out << (SourceMeshItem)item;
  return out;
}



QDataStream&
operator>>(QDataStream &in, SinusoidalSource& item)
{
  SourceMeshItem tmpItem;
  in >> tmpItem;
  (SourceMeshItem)item = tmpItem;
  return in;
}



QDebug
operator<<(QDebug dbg,const SinusoidalSource& item)
{
  dbg.nospace() << "\nSinusoidal Dielectric Source Item:"
      << (SourceMeshItem)item;
  return dbg.maybeSpace();
}


