/*******************************************************************************
**  DESCRIPTION:
**
**
**
**  AUTHOR:         Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:     Dr. Poman So
**
*******************************************************************************/

#ifndef ANIMATORGLWIDGET_H
#define ANIMATORGLWIDGET_H

#include "UI/Graphics/gldisplaywidget.h"
#include "size3d.h"
#include "UI/Graphics/trackball.h"

class MeshEngine;
class MeshSimulator;

class AnimatorGLWidget : public GLDisplayWidget
{
    Q_OBJECT
private:
  MeshEngine*                   meshEngine_;
  TrackBall                     trackBall_;
  QReadWriteLock*               meshEngineMutex_;
  QVector< GLfloat >            surfacePoints_;
  QVector< GLuint >             surfaceIndices_;
  QVector< GLfloat>             surfaceColors_;
  MeshSize                      meshSize_;
  Size3D<float>                 nodeSize_;
  float                         distanceExponent_;
  float                         defaultDistance_;
  float                         maxVoltage_;

public:
  explicit AnimatorGLWidget(QWidget *parent = 0);
  ~AnimatorGLWidget();

  virtual void setupViewingVolume(QSize viewSize) const;
  virtual void initGL();
  virtual void draw() const;

protected:
  void mouseMoveEvent(QMouseEvent *event);
  void mousePressEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);
  void wheelEvent(QWheelEvent *event);

private:
  QPointF pixelPosToViewPos(const QPointF& p);
  void generateSurface();
  void generateIndices();

signals:

public slots:
  void updateAnimation(MeshEngine* meshEngine);
//  void meshEngineChanged(MeshEngine* meshEngine);

};

#endif // ANIMATORGLWIDGET_H
