/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef DATAVIEWER_H
#define DATAVIEWER_H

#include <QWidget>
#include <QtGui>
#include <QGLWidget>
#include "dataset.h"

class DataTransform;
class Mesh;

class DataViewer : public QWidget
{
  Q_OBJECT
private:
  QGraphicsScene*       graphScene_;
  QGraphicsView*        graphView_;
  DataSet               rootData_;
  DataSet               transformedData_;
  DataTransform*        transform_;
  QList<DataTransform*> transforms_;
  bool                  transforming_;
  float                 zoomFactor_;
  QMenu*                contextMenu_;
  QVector2D             graphPadding_;

protected:
  const Mesh*           mesh_;
  QGridLayout*          mainLayout_;
  QComboBox*            transformCombo_;
  QPushButton*          transformSettingsButton_;
  QHBoxLayout*          viewerToolLayout_;


public:
  explicit DataViewer(const Mesh* mesh, QWidget *parent = 0);
  ~DataViewer();

  void setData();
  void setData(QVector<float>& data);
  const Mesh* mesh() const;

protected:
  void createTransformCombo();
  void resizeEvent(QResizeEvent *event);
  void changeEvent(QEvent *event);
  void contextMenuEvent(QContextMenuEvent *event);
  virtual void graph(QGraphicsScene* scene);
  virtual void drawBackground(QGraphicsScene* scene);


private:
  void resizeView();
  void createContextMenu();
  void writeFileHeader(QTextStream& out);

signals:

private slots:
  void draw();
  void changeTransform(int transformIndex);
  void changeTransformSettings();
  void saveData();
  void saveImage();

public slots:
  void updateGraph();

};

#endif // DATAVIEWER_H
