/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef GLDISPLAYWIDGET_H
#define GLDISPLAYWIDGET_H
#include <QGLWidget>
#include <QtCore>


class GLLight
{
public:
  GLenum glLight;
  float pos[4];
  float diffuse[4];
  float specular[4];
  float ambient[4];
public:
  GLLight()
  {
    pos[0] = 0; pos[1] = 0; pos[2] = 0; pos[3] = 0;
    diffuse[0] = 0; diffuse[1] = 0; diffuse[2] = 0; diffuse[3] = 0;
    specular[0] = 0; specular[1] = 0; specular[2] = 0; specular[3] = 0;
    ambient[0] = 0; ambient[1] = 0; ambient[2] = 0; ambient[3] = 0;

  }
  GLLight(GLLight& other)
  {
    glLight = other.glLight;
    for(int i=0;i<4;++i){
      pos[i] = other.pos[i];
      diffuse[i] = other.diffuse[i];
      specular[i] = other.specular[i];
      ambient[i] = other.ambient[i];
    }
  }
  ~GLLight(){}

};


class GLDisplayWidget : public QGLWidget
{
  Q_OBJECT

private:
  QList<GLLight*>       lights_;
  QSize                 viewPortSize_;

protected:
  bool                  blockGLUpdate_;


public:
  ~GLDisplayWidget();

  void setViewPortSize(const QSize newSize);
  const QSize viewPortSize() const;

  void initialize();
  virtual void initGL();
  virtual void setupGLLightingModel();
  QList<GLLight*> getLights() const;


  virtual void setupViewingVolume(QSize viewSize) const = 0;
  virtual void draw() const = 0;


protected:
  explicit GLDisplayWidget(QWidget *parent = 0);
  virtual void initializeGL();
  virtual void closeEvent(QCloseEvent *event);
  virtual void resizeGL(int w, int h);
  virtual void paintGL();
  void addLight(GLLight* light);
  void removeLight(GLLight* light);
  void resizeViewPort(QSize size);
  virtual GLint pick(const QPoint& pos) const;

signals:
  void viewPortResized(QSize size);

public slots:
  void blockGLUpdate(const bool block);


};

#endif // GLDISPLAYWIDGET_H
