/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef OPENGLSCENE_H
#define OPENGLSCENE_H

#include <QGraphicsScene>
#include <QTime>
//#include "point3d.h"
#include "qtoclmesh.h"

class OpenGLScene : public QGraphicsScene
{
    Q_OBJECT
private:
    QTime m_time;
    int m_lastTime;
    int m_mouseEventTime;

    float modelDistance;
    Point3D<float> modelRotation;

    QColor modelColor;
    QColor backgroundColor;

    QMutex* bufferMutex;

    // vertex, index and color BUFFERS
    QVector<float>* meshPoints;
    QVector<int>* meshIndices;
    QVector<float>* meshPointColors;
    //    QVector<Point3d> meshNormals;
    //    QVector<int> m_edgeIndices;


public:
    explicit OpenGLScene(QMutex& initBufferMutex,QObject *parent = 0);
    ~OpenGLScene();

    void drawBackground(QPainter *painter, const QRectF &rect);

private:
    void renderMesh();
    void initialize();

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void wheelEvent(QGraphicsSceneWheelEvent * wheelEvent);

signals:

public slots:
    void updateBuffers(QVector<float>* points, QVector<int>* indices, QVector<float>* colors,MeshSize meshSize);
    void setModelColor();
    const QColor getModelColor()const
    {
	return modelColor;
    }
    void setBackgroundColor();
    const QColor getBackgroundColor() const
    {
	return backgroundColor;
    }
};

#endif // OPENGLSCENE_H
