/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef EDITORVIEW_H
#define EDITORVIEW_H

#include <QWidget>
#include <QtGui>
#include <QGLWidget>
#include "point3d.h"

class Editor;
class MeshViewGL;
class PointMapperGL;
class Mesh;
class MeshItem;

class EditorView : public QWidget
{
    Q_OBJECT

protected:
  Editor*         editor_;
  PointMapperGL&  pointMapper_;
  QScrollBar*     scrollH_;
  QScrollBar*     scrollV_;
  MeshViewGL*     viewGL_;

private:
  QGridLayout*    mainLayout_;
  QHBoxLayout*    glLayout_;
  QSpacerItem*    spacerItem;




public:
  explicit EditorView(Editor *parent);

  void setCameraDistance(const GLfloat distance);
  void setCameraPos(const Point3D<GLfloat> cameraPos,
                    const Point3D<float> changeVector =
                    Point3D<float>(1, 1, 1));
  void moveCamera(const Point3D<GLfloat> deltaPos);

  void setRotation(const Point3D<GLfloat> newRotation);
  void rotateAroundX(GLfloat angle);
  void rotateAroundY(GLfloat angle);
  void rotateAroundZ(GLfloat angle);

  const QSize viewPortSize() const;

  MeshItem* pickItem(const QPoint& pos);

  GLfloat getCameraDistance() const;
  Point3D<GLfloat> getCameraPos() const;
  Point3D<GLfloat> getRotation() const;

  virtual void setupGLView(QSize viewSize);

protected:
  void keyPressEvent(QKeyEvent *event);

private:

signals:
  void hScrollChanged(int value);
  void vScrollChanged(int value);
  void renderMeshItems(Mesh* mesh);

public slots:
  virtual void sceneDepthsChanged(Point3D<float> depths);
  void setHScrollValue(int value);
  void setVScrollValue(int value);
  void centerHScrollBar();
  void centerVScrollBar();
  void sendScrollHEvent(QEvent* event);
  void sendScrollVEvent(QEvent* event);


private slots:
  void updateScrollRange(QSize viewPortSize);

};

#endif // EDITORVIEW_H
