/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "editorview3d.h"
#include "pointmappergl.h"
#include "meshviewgl.h"

EditorView3D::EditorView3D(Editor* parent) :
    EditorView(parent),
    elevation_  (0.7854),
    azimuth_    (0.7854),
    distance_   (pointMapper_.worldSize()/2)
{
  viewGL_->installEventFilter((QObject*)(this));
  scrollH_->setDisabled(true);
  scrollV_->setDisabled(true);

//  viewGL_->setCameraUp(vAxis_);
//  viewGL_->setCameraAt(Point3D<GLfloat>(0,0,0));
//  viewGL_->setCameraPos(crossPoint.mask()*(pointMapper_.worldSize()/2.0));

  adjustCameraPosition();
}



void
EditorView3D::setupGLView(QSize viewSize)
{
  int halfWorld = pointMapper_.mapGLToPixel(0);
  float orthoWidth = pointMapper_.mapPixelToGL(halfWorld+viewSize.width());
  float orthoHeight = pointMapper_.mapPixelToGL(halfWorld+viewSize.height());
  gluPerspective(20,(float)(viewSize.width())/viewSize.height(),0.001,10000);
//  glOrtho(-orthoWidth/2.0, orthoWidth/2.0, -orthoHeight/2.0, orthoHeight/2.0,
//          pointMapper_.drawingResolution(), pointMapper_.worldSize());
//    glOrtho(0, orthoWidth, 0, orthoHeight,
//            pointMapper_.drawingResolution(), 100);
}



void
EditorView3D::adjustElevation(const float deltaElevation)
{
  elevation_ += deltaElevation;
  adjustCameraPosition();
}



void
EditorView3D::adjustAzimuth(const float deltaAzimuth)
{
  azimuth_ += deltaAzimuth;
  adjustCameraPosition();
}



void
EditorView3D::adjustDistance(const float deltaDistance)
{
  distance_ += deltaDistance;
  adjustCameraPosition();
}



void
EditorView3D::setElevation(const float elevation)
{
  elevation_ = elevation;
  adjustCameraPosition();
}



void
EditorView3D::setAzimuth(const float azimuth)
{
  azimuth_ = azimuth;
  adjustCameraPosition();
}



void
EditorView3D::setDistance(const float distance)
{
  distance_ = distance;
  adjustCameraPosition();
}






//-----------------------------------------------------------------------------
//  PROTECTED METHODS
//-----------------------------------------------------------------------------

bool
EditorView3D::eventFilter(QObject *object, QEvent *event)
{
  // Mouse Move
  if(event->type() == QEvent::MouseMove){
    QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(event);
    QPoint mousePos = mouseEvent->pos();
//    emit mouseMoved(mousePos,mouseEvent->buttons(),mouseEvent->modifiers());
    QPoint deltaPos = mouseCurrentPos - mouseEvent->pos();
    mouseCurrentPos = mouseEvent->pos();

    Point3D<GLfloat> currentRotation = getRotation();

    if(mouseEvent->buttons().testFlag(Qt::LeftButton)){
      currentRotation.ry() += 180.0*deltaPos.x()/viewPortSize().width();
      currentRotation.rz() += 180.0*deltaPos.y()/viewPortSize().height();
    }
    else if(mouseEvent->buttons().testFlag(Qt::MiddleButton)){
      currentRotation.rx() += 180.0*deltaPos.y()/viewPortSize().height();
    }

    adjustElevation(6.0*deltaPos.y()/viewPortSize().height());
    adjustAzimuth(6.0*deltaPos.x()/viewPortSize().height());
    return true;
  }
  // Mouse Press
  if(event->type() == QEvent::MouseButtonPress){
    QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(event);
    QPoint mousePos = mouseEvent->pos();
    emit mousePressed(mousePos,mouseEvent->buttons(),mouseEvent->modifiers());
    return true;
  }
//  // Mouse Release
//  else if(event->type() == QEvent::MouseButtonRelease){
//    QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(event);
//    QPoint mousePos = mouseEvent->pos();
//    emit mouseReleased(mousePos,mouseEvent->buttons(),mouseEvent->modifiers());
//    return true;
//  }
  return false;
}



//void
//EditorView3D::mouseMoveEvent(QMouseEvent *mouseEvent)
//{
//  QPoint deltaPos = mouseCurrentPos - mouseEvent->pos();
//  mouseCurrentPos = mouseEvent->pos();


//  Point3D<GLfloat> currentRotation = getRotation();

//  if(mouseEvent->buttons().testFlag(Qt::LeftButton)){
//    currentRotation.ry() += 180.0*deltaPos.x()/viewPortSize().width();
//    currentRotation.rz() += 180.0*deltaPos.y()/viewPortSize().height();
//  }
//  else if(mouseEvent->buttons().testFlag(Qt::MiddleButton)){
//    currentRotation.rx() += 180.0*deltaPos.y()/viewPortSize().height();
//  }

//  adjustElevation(6.0*deltaPos.y()/viewPortSize().height());
//  adjustAzimuth(6.0*deltaPos.x()/viewPortSize().height());
//}



//void
//EditorView3D::mousePressEvent(QMouseEvent *mouseEvent)
//{
//  mouseCurrentPos = mouseEvent->pos();
//  mouseStartPos = mouseCurrentPos;
//  MeshItem* pickedItem = viewGL_->pickItem(mouseCurrentPos);
//  if(pickedItem != 0){
//    qDebug()<<"Picked Item:"<<pickedItem;
//  }

//}



//void
//EditorView3D::mouseReleaseEvent(QMouseEvent *mouseEvent)
//{

//}



void
EditorView3D::wheelEvent(QWheelEvent *wheelEvent)
{
   adjustDistance(-(wheelEvent->delta()/360.0)*pointMapper_.worldSize()/10.0);
}



//-----------------------------------------------------------------------------
//  PRIVATE METHODS
//-----------------------------------------------------------------------------


void
EditorView3D::adjustCameraPosition()
{
//  qDebug()<<"Elevation:"<<elevation_;
//  qDebug()<<"Azimuth:"<<azimuth_;
//  qDebug()<<"distance:"<<distance_;
  Point3D<GLfloat> adjustedCamera, adjustCamUp;
  adjustedCamera.rx() = distance_*sin(elevation_)*cos(azimuth_);
  adjustedCamera.ry() = distance_*sin(elevation_)*sin(azimuth_);
  adjustedCamera.rz() = distance_*cos(elevation_);
  viewGL_->setCameraPos(adjustedCamera);
//  viewGL_->setCameraUp();
}
