/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "meshitemlistselector.h"
#include "meshitems.h"

#define QVARIANT_FLOAT  135

MeshItemListSelector::MeshItemListSelector(QList<MeshItem *> itemList, QWidget *parent) :
    MeshItemSelector(itemList,parent)
{
  mainLayout_ = new QVBoxLayout;
  setLayout(mainLayout_);
  createMeshItemView();

  connect(itemListWidget, SIGNAL(itemChanged(QTreeWidgetItem*,int)),
          this, SLOT(meshItemListEdited(QTreeWidgetItem*,int)));
  connect(itemListWidget, SIGNAL(itemSelectionChanged()),
          this, SLOT(meshItemListSelect()));

  mainLayout_->addWidget(itemListWidget);
}




QTreeWidgetItem*
MeshItemListSelector::createMeshPropertyListItem(QTreeWidgetItem* parentItem,
MeshItemProperty meshItemProperty) const
{
  QString id = meshItemProperty.name();
  QTreeWidgetItem* propertyListItem = new QTreeWidgetItem(parentItem);
  propertyListItem->setFont(0,QFont(QString("MS Shell Dlg 2")));
  propertyListItem->setTextAlignment(0, Qt::AlignLeft|Qt::AlignVCenter);
  // Set title
  propertyListItem->setText(0,meshItemProperty.title());
  propertyListItem->setToolTip(0,id.section("/",-1));
  // Set ID of list item
  propertyListItem->setText(2,id);
  // Set value of the property
  if(meshItemProperty.defaultValue().type()==QVariant::Bool){
    propertyListItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
    if(meshItemProperty.defaultValue().toBool()==true){
      propertyListItem->setCheckState(0,Qt::Checked);
    }else{
      propertyListItem->setCheckState(0,Qt::Unchecked);
    }
  }else{
    propertyListItem->setFlags(Qt::ItemIsEditable | Qt::ItemIsEnabled);
    if(meshItemProperty.defaultValue().type() == QVARIANT_FLOAT){ // Equivilent to QVariant::float since it doesn't exist
      propertyListItem->setText(1,tr("%1").
                                arg(meshItemProperty.defaultValue().toFloat()));
    }
    else if(meshItemProperty.defaultValue().type() == QVariant::Int){
      propertyListItem->setText(1,tr("%1").
                                arg(meshItemProperty.defaultValue().toInt()));
    }
  }
  return propertyListItem;
}



QTreeWidgetItem*
MeshItemListSelector::createMeshItemListItem(QTreeWidgetItem* parent,
MeshItem* item,
QList<MeshItemSelector::MeshItemProperty>& commonProperties) const
{
  QSize iconSize(25,25);
  QString ID = QString(item->metaObject()->className());
  QTreeWidgetItem* newListItem = new QTreeWidgetItem(parent);
  newListItem->setFont(0,QFont(QString("MS Shell Dlg 2"),-1,QFont::Bold));
  newListItem->setTextAlignment(0, Qt::AlignLeft|Qt::AlignVCenter);
  newListItem->setText(0,item->getDisplayText());
  newListItem->setToolTip(0,item->getToolTip());
  newListItem->setWhatsThis(0,item->getWhatsThis());
  newListItem->setIcon(0,QIcon(item->generateImage(iconSize,
                                                 item->getBorderColor(),
                                                 item->getFillColor())));
  newListItem->setText(2,ID);
  newListItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);

  QVariant meshListItemData;
  meshListItemData.setValue((void*)item);
  newListItem->setData(3,Qt::UserRole,meshListItemData);

  QList<MeshItemSelector::MeshItemProperty> itemProperties =  getItemProperties(item);
  foreach(MeshItemSelector::MeshItemProperty property, itemProperties){
    if(!commonProperties.contains(property)){
      createMeshPropertyListItem(newListItem,property);
    }
  }
  return newListItem;
}



QTreeWidgetItem*
MeshItemListSelector::createCommonPropertiesListItem(QTreeWidgetItem *parent,
QList<MeshItemSelector::MeshItemProperty> &commonProperties) const
{
  QTreeWidgetItem* commonPropertyListItem = 0;
  if(commonProperties.count()>0){
    QString ID = parent->text(2).append(QString("/properties"));
    QTreeWidgetItem* commonPropertyListItem = new QTreeWidgetItem(parent);
    commonPropertyListItem->setFont(0,QFont(QString("MS Shell Dlg 2"),-1,-1,true));
    commonPropertyListItem->setTextAlignment(0, Qt::AlignLeft|Qt::AlignVCenter);
    commonPropertyListItem->setText(0,QString("Properties"));
    commonPropertyListItem->setToolTip(0,QString("Common Properties"));
    commonPropertyListItem->setWhatsThis(0,QString("Common Properties for this "
                                                   "particular type of item."));
    commonPropertyListItem->setFirstColumnSpanned(true);
    commonPropertyListItem->setText(2,ID);
    commonPropertyListItem->setFlags(Qt::ItemIsEnabled);

    foreach(MeshItemSelector::MeshItemProperty property, commonProperties){
      createMeshPropertyListItem(commonPropertyListItem,property);
    }
  }
  return commonPropertyListItem;
}




void
MeshItemListSelector::createMeshItemView(){


  // LIST INITIALIZATION
  itemListWidget = new QTreeWidget();

  QTreeWidgetItem * treeHeader = itemListWidget->headerItem();
  treeHeader->setText(0,tr("Item"));
  treeHeader->setText(1,tr("Value"));
  treeHeader->setText(2,tr("ID"));
  treeHeader->setText(3,tr("Object"));

  //itemList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  itemListWidget->setAlternatingRowColors(true);
  //  itemList->setTextElideMode(Qt::ElideRight);
  itemListWidget->setHorizontalScrollMode(QAbstractItemView::ScrollPerItem);
  itemListWidget->setIndentation(8);
  itemListWidget->setUniformRowHeights(false);
  itemListWidget->setAnimated(true);
  itemListWidget->setColumnCount(4);
  //  itemList->header()->setVisible(false);
  itemListWidget->header()->setCascadingSectionResizes(true);
  itemListWidget->header()->setDefaultSectionSize(100);
  itemListWidget->header()->setHighlightSections(true);
  itemListWidget->header()->setMinimumSectionSize(10);
  itemListWidget->header()->setStretchLastSection(false);


  itemListWidget->setAutoScroll(false);
  //  itemList->hideColumn(2);
  //  itemList->hideColumn(3);



  QList<MeshItem*> supportedItems = itemList_;
  QList<MeshItemSelector::MeshItemProperty> commonProperties;
  // Get the lists of items sorted by type


  QList<MeshItem*> boundaryItems =
      getMeshItemsByType<BoundaryMeshItem>(supportedItems);
  if(boundaryItems.count()>0){
    commonProperties = getCommonProperties(boundaryItems);
    QTreeWidgetItem * boundaryRoot= new QTreeWidgetItem(itemListWidget,0);
    boundaryRoot->setText(0,tr("Boundary"));
    boundaryRoot->setText(2,"boundary");
    boundaryRoot->setFlags(Qt::ItemIsEnabled);

    createCommonPropertiesListItem(boundaryRoot,commonProperties);
    foreach(MeshItem* item, boundaryItems){
      createMeshItemListItem(boundaryRoot,item,commonProperties);
    }
    itemListWidget->expandItem(boundaryRoot);
  }


  QList<MeshItem*> materialItems =
      getMeshItemsByType<MaterialMeshItem>(supportedItems);
  if(materialItems.count()>0){
    commonProperties = getCommonProperties(materialItems);
    QTreeWidgetItem * materialRoot= new QTreeWidgetItem(itemListWidget,0);
    materialRoot->setText(0,tr("Material"));
    materialRoot->setText(2,QString("material"));
    materialRoot->setFlags(Qt::ItemIsEnabled);
    createCommonPropertiesListItem(materialRoot,commonProperties);
    foreach(MeshItem* item, materialItems){
      createMeshItemListItem(materialRoot,item,commonProperties);
    }
    itemListWidget->expandItem(materialRoot);
  }


  QList<MeshItem*> probeItems =
      getMeshItemsByType<ProbeMeshItem>(supportedItems);
  if(probeItems.count()>0){
    commonProperties = getCommonProperties(probeItems);
    QTreeWidgetItem * probeRoot= new QTreeWidgetItem(itemListWidget,0);
    probeRoot->setText(0,tr("Probes"));
    probeRoot->setText(2,QString("probes"));
    probeRoot->setFlags(Qt::ItemIsEnabled);
    createCommonPropertiesListItem(probeRoot,commonProperties);
    foreach(MeshItem* item, probeItems){
      createMeshItemListItem(probeRoot,item,commonProperties);
    }
    itemListWidget->expandItem(probeRoot);
  }


  QList<MeshItem*> sourceItems =
      getMeshItemsByType<SourceMeshItem>(supportedItems);
  if(sourceItems.count()>0){
    commonProperties = getCommonProperties(sourceItems);
    QTreeWidgetItem * sourceRoot= new QTreeWidgetItem(itemListWidget,0);
    sourceRoot->setText(0,tr("Sources"));
    sourceRoot->setText(2,QString("sources"));
    sourceRoot->setFlags(Qt::ItemIsEnabled);
    createCommonPropertiesListItem(sourceRoot,commonProperties);
    foreach(MeshItem* item, sourceItems){
      createMeshItemListItem(sourceRoot,item,commonProperties);
    }
    itemListWidget->expandItem(sourceRoot);
  }

}



QMap<QString,QVariant>
MeshItemListSelector::getListItemProperties(QTreeWidgetItem* selectedItem) const
{
  QMap<QString,QVariant> propertyMap;
//    QString searchString = selectedItem->parent()->text(2);
//    searchString.append(QString("_properties"));

//    QList<QTreeWidgetItem*> propertyList =
//      ui->itemList->findItems(searchString,
//            Qt::MatchExactly|Qt::MatchRecursive,2);
//    if(propertyList.count()==1){
//  QTreeWidgetItem * properties = propertyList.first();
//  // Get the properties associated with this mesh Item
//  // and set the values of the new object with the values
//  // that are currently in the property fields.
//  for(int i=0;i<properties->childCount();++i){
//      QString propertyName = properties->child(i)->text(2);
//      QVariant propertyValue = getListItemPropertyValue(properties->child(i));
//      propertyMap.insert(propertyName,propertyValue);
//  }
//    }
  return propertyMap;
}



QVariant
MeshItemListSelector::getListItemPropertyValue(QTreeWidgetItem *propertyListItem) const
{
  QVariant propertyValue;
  if(propertyListItem->flags().testFlag(Qt::ItemIsUserCheckable)){
    propertyValue = QVariant::fromValue(propertyListItem->checkState(0) == Qt::Checked);
  }else{
    float floatTest = propertyListItem->text(1).toFloat();
    // Test to see if this value is an integer by rounding it and comparing it
    // to the original.
    if(qRound(floatTest) == floatTest){
      propertyValue = QVariant::fromValue(propertyListItem->text(1).toInt());
    }else{
      propertyValue = QVariant::fromValue(propertyListItem->text(1).toFloat());
    }
  }
  return propertyValue;
}



void
MeshItemListSelector::meshItemListSelect(){
  QList<QTreeWidgetItem*> selectList;
  selectList = itemListWidget->selectedItems();
  if(selectList.count()==1){
    QTreeWidgetItem* selectedItem = selectList.first();
    MeshItem* newMeshItem = ((MeshItem*)selectedItem->data(3,Qt::UserRole).value<void*>());
    emit itemSelected(newMeshItem);
  }
}



void
MeshItemListSelector::meshItemListEdited(QTreeWidgetItem *listItem, int col){
    QString propertyName = listItem->text(2);
    QVariant propertyValue = getListItemPropertyValue(listItem);
    foreach(MeshItem* item, itemList_){
      item->setPropertyValue(propertyName,propertyValue);
    }
    emit propertyChanged(propertyName,propertyValue);
}




void
MeshItemListSelector::syncItemViewProperties(QList<MeshItem *> items)
{

}



void
MeshItemListSelector::clearItemViewSelection()
{
  itemListWidget->clearSelection();
}
