/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/


#include "meshitemmanager.h"
// COMMANDS - UNDO/REDO FRAMEWORK
#include "commands.h"
#include "meshscene.h"
#include "meshgraphicsitem.h"
#include "meshgraphicsitemgroup.h"
#include "DataContainers/coordinatemapper.h"
#include "qtoclmesh.h"


MeshItemManager::MeshItemManager(QList<MeshScene*>& initSceneList,
                                 QSharedPointer<CoordinateMapper>initMapper) :
    QObject(0),
    meshSceneList(&initSceneList),
    meshItemContextMenu(NULL),
    selectedItemGroup(QSharedPointer<MeshGraphicsItemGroup>(new MeshGraphicsItemGroup(initMapper))),
    pointMapper(initMapper),
    undoStack(QSharedPointer<QUndoStack>(new QUndoStack()))
{
    meshItem = QSharedPointer<MeshItem>(new MeshItem);
    connectSignals();
    createMeshItemContextMenu();
}


MeshItemManager::~MeshItemManager(){
    selectedItemGroup.clear();
    pointMapper.clear();
    undoStack.clear();
    meshItem.clear();
}


void MeshItemManager::connectSignals(){
    connect(selectedItemGroup.data(),SIGNAL(signalSelectionChanged(QList<MeshGraphicsItem*>)),this,SLOT(selectionChanged(QList<MeshGraphicsItem*>)));
}

void MeshItemManager::createMeshItemContextMenu()
{
    meshItemContextMenu = new QMenu;

    actionUndo = undoStack->createUndoAction(this,tr("&Undo"));
    actionUndo->setShortcuts(QKeySequence::Undo);
    actionUndo->setIcon(QIcon(":Images/Images/undo.png"));

    actionRedo = undoStack->createRedoAction(this,tr("&Redo"));
    actionRedo->setShortcuts(QKeySequence::Redo);
    actionRedo->setIcon(QIcon(":Images/Images/redo.png"));

    actionMoveItem = new QAction(tr("&Move"),this);

    actionScaleItem = new QAction(tr("&Scale"),this);
    actionScaleItem->setIcon(QIcon(":/Images/Images/scaleNormalOn.png"));

    actionDeleteItem = new QAction(tr("&Delete"),this);
    actionDeleteItem->setShortcuts(QKeySequence::Delete);
    actionDeleteItem->setIcon(QIcon(":/Images/Images/deleteNormalOn.png"));

    meshItemContextMenu->addAction(actionUndo);
    meshItemContextMenu->addAction(actionRedo);
    meshItemContextMenu->addSeparator();
    meshItemContextMenu->addAction(actionMoveItem);
    meshItemContextMenu->addAction(actionScaleItem);
    meshItemContextMenu->addAction(actionDeleteItem);
}

QMenu* MeshItemManager::getMeshItemContextMenu()const
{
    return meshItemContextMenu;
}

void MeshItemManager::setMeshItem(MeshItem* item)
{
    meshItem.clear();
    meshItem = QSharedPointer<MeshItem>(item->clone());
    QRectF initialMeshItemRect(QPointF(0,0),QSizeF(pointMapper->getSceneUnitsPerNode(),pointMapper->getSceneUnitsPerNode()));
    meshItem->setSceneRect(initialMeshItemRect);
}


void MeshItemManager::selectItems(QList<MeshGraphicsItem*> items){
    MeshGraphicsItem* itemToSelect = items.first();
    foreach(MeshGraphicsItem* itemCheck,items){
	if(itemCheck->getSceneSize().width()<itemToSelect->getSceneSize().width())
	    itemToSelect = itemCheck;
	if(itemCheck->getSceneSize().height()<itemToSelect->getSceneSize().height())
	    itemToSelect = itemCheck;
    }
    selectedItemGroup->addToGroup(itemToSelect);
}


void MeshItemManager::clearSelection(){
    selectedItemGroup->removeAllFromGroup();
}

const QList<MeshGraphicsItem*> MeshItemManager::selectedItems()const{
    return selectedItemGroup->children();
}

const bool MeshItemManager::selectionHitTest(const QPointF point) const{
    bool hit = false;
//    foreach(QGraphicsItem* item,meshScene->items(point)){
//        if(item==selectedItemGroup.data()){
//            hit = true;
//            break;
//        }
//    }
    return hit;
}

void MeshItemManager::update(const QRectF rect){
    selectedItemGroup->update(rect);
}

void MeshItemManager::cleanMeshItemList(QList<MeshGraphicsItem *> &itemList){
    foreach(MeshGraphicsItem* item,itemList){
        QGraphicsItem* itemPtr = (QGraphicsItem*)item;
        if(itemPtr == selectedItemGroup.data()){
            itemList.removeOne(item);
            break;
        }
    }
}

void MeshItemManager::insertItem(){
//    MeshGraphicsItem* newItem = new MeshGraphicsItem(meshItem,pointMapper,meshScene);
//    selectedItemGroup->addToGroup(newItem);
//    emit meshItemInserted(newItem);
//    emit meshModified();
}

// ITEM MODIFIERS
// These functions are responsible for modifying items by modifying
// the selectedItemGroup which in turn applies the transforms to the
// items contained within it.

// Moves items around the scene
void MeshItemManager::setScenePos(const QPointF point){
    if(selectedItemGroup->children().count()>0){
        selectedItemGroup->setPos(pointMapper->snapSceneToMesh(point));
    }else{
        meshItem->setScenePos(pointMapper->snapSceneToMesh(point));
    }
    emit meshModified();
}

// Adjusts the scale of the items by a delta size
void MeshItemManager::adjustSize(QSizeF deltaSize){
    selectedItemGroup->adjustSize(deltaSize);
    emit meshModified();
}

void MeshItemManager::shiftItems(const QPointF deltaPos)
{
    if(selectedItemGroup->children().count()>0){
        QPointF newPosition = selectedItemGroup->scenePos()+deltaPos;
        qDebug()<<"NewPosition:"<<newPosition;
        selectedItemGroup->setPos(pointMapper->snapSceneToMesh(newPosition));
    }/*else{
        newPosition = meshItem->getSceneRect
        meshItem->setScenePos(pointMapper->snapSceneToMesh(newPosition));
    }*/
    emit meshModified();
}

void MeshItemManager::setItemProperty(QString propertyName, QVariant propertyValue){
    if(selectedItemGroup->children().count()>0){
	foreach(MeshGraphicsItem* item,selectedItemGroup->children()){
	    item->setPropertyValue(propertyName,propertyValue);
	}
    }
}


// SLOTS
void MeshItemManager::deleteItems(){
//    foreach(MeshGraphicsItem* item,selectedItems()){
//        meshScene->removeItem(item);
//    }
//    clearSelection();
//    emit meshModified();
}

void MeshItemManager::selectionChanged(QList<MeshGraphicsItem*> itemList){
//    SafeFloat reflectionCoef,conductance,permittivity;
//    foreach(MeshGraphicsItem* item,itemList){
//	switch(item->getProperties().getItemType()){
//	case ItemProperties::Boundary:
//	    reflectionCoef.setValue(item->getProperties().getReflectionCoef().getValue());
//	    qDebug()<<"Reflection: "<<reflectionCoef.getValue();
//	    break;
//	case ItemProperties::Material:
//	    conductance.setValue(item->getProperties().getConductance().getValue());
//	    permittivity.setValue(item->getProperties().getRelPermittivity().getValue());
//	    qDebug()<<"Conductance: "<<conductance.getValue()<<"permittivity: "<<permittivity.getValue();
//	    break;
//	}
//    }

//    if(reflectionCoef.isInitialized())
//	emit currentReflectionCoefChanged(reflectionCoef);
//    if(permittivity.isInitialized())
//	emit currentRelPermittivityChanged(permittivity);
//    if(conductance.isInitialized())
//	emit currentConductanceChanged(conductance);
}
