#ifndef MESHITEMMANAGER_H
#define MESHITEMMANAGER_H
#include <QObject>
#include <QtCore>
#include <QtGui>
//#include "DataContainers/safefloat.h"
//#include "meshgraphicsitemgroup.h"
//#include "meshgraphicsitem.h"
//#include "DataContainers/coordinatemapper.h"
//#include "DataContainers/meshitemshape.h"
//#include "DataContainers/itemproperties.h"
//#include "UI/Windows/mainwindow.h"
//#include "meshitems.h"

class MeshScene;
class CoordinateMapper;
class MeshItem;
class MeshGraphicsItem;

class MeshItemManager : public QObject
{
Q_OBJECT

public:

private:
    QList<MeshScene*>* meshSceneList;

    QSharedPointer<MeshGraphicsItemGroup> selectedItemGroup;
    QSharedPointer<CoordinateMapper> pointMapper;
    QSharedPointer<QUndoStack> undoStack;
    QSharedPointer<MeshItem> meshItem;

public:
    explicit MeshItemManager(QList<MeshScene*>& initSceneList,QSharedPointer<CoordinateMapper>initMapper);
    ~MeshItemManager();

    QMenu* getMeshItemContextMenu()const;


    void setMeshItem(MeshItem* item);

    const bool selectionHitTest(const QPointF point) const;

    const bool isSelected(MeshGraphicsItem* item)const{return selectedItemGroup->isInGroup(item);}
    const bool isSelected(QList<MeshGraphicsItem*> items)const{return selectedItemGroup->areInGroup(items);}
    void selectItem(MeshGraphicsItem* item){selectedItemGroup->addToGroup(item);}
    void selectItems(QList<MeshGraphicsItem*> items);
    void deselectItem(MeshGraphicsItem* item){selectedItemGroup->removeFromGroup(item);}
    void deselectItems(QList<MeshGraphicsItem*> items){selectedItemGroup->removeFromGroup(items);}
    void clearSelection();

    const QList<MeshGraphicsItem*> selectedItems() const;

    void cleanMeshItemList(QList<MeshGraphicsItem*>& itemList);

    void insertItem();
    void setScenePos(const QPointF pos);
    void adjustSize(const QSizeF deltaSize);
    void shiftItems(const QPointF deltaPos);
    void setItemProperty(QString propertyName,QVariant propertyValue);

    void update(const QRectF rect);

    const QPointF getSelectionPos()const
    {
        return selectedItemGroup->scenePos();
    }
    const QRectF getSelectionRect()const
    {
        return QRectF(selectedItemGroup->scenePos().rx()+selectedItemGroup->boundingRect().left(),
                             selectedItemGroup->scenePos().ry()+selectedItemGroup->boundingRect().top(),
                             selectedItemGroup->boundingRect().width(),
                             selectedItemGroup->boundingRect().height());
    }

    QSharedPointer<QUndoStack> getUndoStackPointer()const
    {
        return undoStack;
    }
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
    {
        //selectedItemGroup->contextMenuEvent(event);
    }

private:
    void connectSignals();
    void createMeshItemContextMenu();

signals:
    void meshItemInserted(MeshGraphicsItem* item);
    void meshModified();

public slots:
    void deleteItems();
    void selectionChanged(QList<MeshGraphicsItem*> itemList);


/********************************************************************************
**			USER INTERFACE COMPONENTS
*********************************************************************************/

private:
    QMenu * meshItemContextMenu;

    QAction* actionUndo;
    QAction* actionRedo;
    QAction* actionDeleteItem;
    QAction* actionMoveItem;
    QAction* actionScaleItem;

};

#endif // MESHITEMMANAGER_H
