/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "meshitemselector.h"
#include "meshitems.h"

MeshItemSelector::MeshItemSelector(const QList<MeshItem *> itemList,
                                   QWidget *parent) :
    QWidget   (parent)
{
  foreach(MeshItem* item, itemList){
    itemList_.append(item->clone());
  }
}


MeshItemSelector::~MeshItemSelector()
{
  qDeleteAll(itemList_);
}




QList<MeshItemSelector::MeshItemProperty>
MeshItemSelector::getCommonProperties(const QList<MeshItem*> items)const
{
  QList<MeshItemSelector::MeshItemProperty> commonProperties;
  if(items.count()>0){
    commonProperties = getItemProperties(items.first());
    QList<MeshItemSelector::MeshItemProperty> itemProperties;
    foreach(MeshItem* item, items){
      itemProperties = getItemProperties(item);
      if(itemProperties.count()>0){
        foreach(MeshItemSelector::MeshItemProperty commonProperty, commonProperties){
          if(!itemProperties.contains(commonProperty)){
            commonProperties.removeOne(commonProperty);
          }
        }
      }
      else{
        commonProperties.clear();
        break;
      }
    }
  }
  return commonProperties;
}



QList<MeshItemSelector::MeshItemProperty>
MeshItemSelector::getItemProperties(const MeshItem* item) const
{
  QList<MeshItemSelector::MeshItemProperty> itemProperties;
  QString propertyTitle, propertyName;
  QVariant propertyValue;
  QMap<QString,QString> itemPropertyMap = item->getPropertyTextMap();
  QMap<QString,QString>::const_iterator i = itemPropertyMap.constBegin();
  while(i != itemPropertyMap.constEnd()){
    propertyTitle = i.value();
    propertyName = i.key();
    propertyValue = item->getPropertyValue(propertyName);
    itemProperties.append(
        MeshItemSelector::MeshItemProperty(propertyName,
                                 propertyTitle,
                                 propertyValue));
    i++;
  }
  return itemProperties;
}


