/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MESHITEMSELECTOR_H
#define MESHITEMSELECTOR_H

#include <QWidget>
#include <QtGUI>

class MeshItem;
class Editor;

class MeshItemSelector : public QWidget
{
  Q_OBJECT
protected:
  class MeshItemProperty
  {
  private:
    QString   name_;
    QString   title_;
    QVariant  defaultValue_;
  public:
    MeshItemProperty(QString name = tr(""),
                     QString title = tr(""),
                     QVariant defaultValue = QVariant(0.0)):
    name_         (name),
    title_        (title),
    defaultValue_ (defaultValue)
    {
    }


    bool operator==(MeshItemProperty other){
      return (name_ == other.name_)&&
          (title_ == other.title_);
    }


    QString name(){ return name_; }
    QString title(){ return title_;}
    QVariant defaultValue(){ return defaultValue_; }

  };
  QList<MeshItem*>  itemList_;
  QLayout*          mainLayout_;
  MeshItem*         selectedItem_;
protected:
  explicit MeshItemSelector(const QList<MeshItem*> itemList,
                              QWidget* parent = 0);
  ~MeshItemSelector();
  template<typename T> QList<MeshItem*>
  getMeshItemsByType(QList<MeshItem*> unsortedList) const
  {
    QList<MeshItem*> filteredList;
    T * filterItem;
    foreach(MeshItem* item, unsortedList){
      filterItem = qobject_cast<T*>((QObject*)item);
      if(filterItem!=0){
        filteredList.append(item);
      }
    }
    return filteredList;
  }
  QList<MeshItemProperty> getCommonProperties(const QList<MeshItem*> items)const;
  QList<MeshItemProperty> getItemProperties(const MeshItem* item) const;

public slots:
  virtual void syncItemViewProperties(QList<MeshItem*> items) = 0;
  virtual void clearItemViewSelection() = 0;

signals:
  void propertyChanged(QString propertyName, QVariant value);
  void itemSelected(MeshItem* item);





};

#endif // MESHITEMSELECTOR_H
