/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MESHVIEWGL_H
#define MESHVIEWGL_H

#include <QGLWidget>
#include "qtoclmesh.h"
#include "UI/Graphics/gldisplaywidget.h"

class MeshItem;
class Mesh;
class PointMapperGL;
class EditorView;
class EditorView2D;



class MeshViewGL : public GLDisplayWidget
{
    Q_OBJECT

private:
  PointMapperGL&        pointMapper_;
  EditorView*           editorView_;
  QList<MeshItem*>      meshItems_;
  Point3D<GLfloat>      cameraPos_;
  Point3D<GLfloat>      cameraAt_;
  Point3D<GLfloat>      cameraUp_;
  Point3D<GLfloat>      cameraRight_;
  Point3D<GLfloat>      rotation_;
  GLfloat               cameraDistance_;


public:

  explicit MeshViewGL(PointMapperGL& pointMapper, EditorView* editorView);
  ~MeshViewGL();

  void setCameraDistance(const GLfloat distance);
  void setCameraPos(const Point3D<GLfloat> cameraPos,
                    const Point3D<float> changeVector =
                    Point3D<float>(1, 1, 1));
  void moveCamera(const Point3D<GLfloat> deltaPos);
  void setCameraAt(const Point3D<GLfloat> cameraAt,
                   const Point3D<float> changeVector =
                   Point3D<float>(1, 1, 1));
  void setCameraUp(const Point3D<GLfloat> cameraUp,
                   const Point3D<float> changeVector =
                   Point3D<float>(1, 1, 1));

  void setRotation(const Point3D<GLfloat> newRotation);
  void rotateAroundX(GLfloat angle);
  void rotateAroundY(GLfloat angle);
  void rotateAroundZ(GLfloat angle);


  GLfloat getCameraDistance() const;
  Point3D<GLfloat> getCameraPos() const;
  Point3D<GLfloat> getCameraAt() const;
  Point3D<GLfloat> getCameraUp() const;
  Point3D<GLfloat> getRotation() const;

  PointMapperGL& getPointMapper() const;
  EditorView* getEditorView() const;

  MeshItem* pickItem(const QPoint& pos);

  virtual void initGL();
  virtual void setupViewingVolume(QSize viewSize) const;
  virtual void setupGLLightingModel();
  virtual void draw() const;


private:
  virtual void drawGrid(EditorView2D * editor) const;
  void initializeLighting();

public slots:
  void renderMesh(Mesh* mesh);
  void zoomFactorChanged();
  void pressPointsChanged();




};

#endif // MESHVIEWGL_H
