/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MOUSETRACKER_H
#define MOUSETRACKER_H

#include <QObject>
#include "point3d.h"

class MouseTracker : public QObject
{
    Q_OBJECT
private:
  Point3D<float>  pressPosition_[2];
  Point3D<float>  movePosition_[2];
  Point3D<float>  releasePosition_;
  bool            showPressPositions_;

public:
  explicit MouseTracker();
  //MouseTracker(MouseTracker& other);
  MouseTracker& operator =(MouseTracker& other);

  void showPressPositions(const bool show);
  void setPressPosition(const Point3D<float> pressPosition,
                        Point3D<float> viewMask);
  void setMovePosition(const Point3D<float> movePosition,
                       Point3D<float> viewMask);
  void setReleasePosition(const Point3D<float> releasePosition,
                          Point3D<float> viewMask);

  const bool pressPositionsVisible() const;
  const Point3D<float>& getLastPressPosition() const;
  const Point3D<float>& getPressPosition() const;
  const Point3D<float>& getMovePosition() const;
  const Point3D<float>& getLastMovePosition() const;
  const Point3D<float>& getReleasePosition() const;

  // Difference between the two mouse press positions
  const Point3D<float> getDeltaPress() const;
  // Difference between the last press position and the current move position
  const Point3D<float> getMovePressDelta() const;
  // Get the ammount that the mouse position has moved since the last
  // set move position call
  const Point3D<float> getMoveDelta() const;
  // Difference between the last press position and the release position
  const Point3D<float> getReleasePressDelta() const;

signals:
  void pressPositionChanged(Point3D<float>& lastPressPosition,
                            Point3D<float>& firstPressPosition);
  void movePositionChanged(Point3D<float>& currentMousePosition,
                           Point3D<float>& lastMousePosition);
  void releasePositionChanged(Point3D<float>& releasePosition);

public slots:

};

#endif // MOUSETRACKER_H
