/*******************************************************************************
**  FILE:	    meshsimulationthread.cpp
**  DESCRIPTION:    Thread used for running the simulation routines in a different
**		    thread (derrived from QThread class).  The simulation thread
**		    makes the appropriate calls to the MeshEngine class which
**		    provides the interface for simulating a Mesh.
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "meshsimulationthread.h"
#include "meshengine.h"


MeshSimulationThread::MeshSimulationThread(QObject *parent) :
    QThread(parent)
{
    delayFactor = 50;  // Delay factor in mSecs
    abort = false;
    pause = false;
}

MeshSimulationThread::~MeshSimulationThread()
{
  stopSimulation();
}



void
MeshSimulationThread::simulate(MeshEngine *engine,
                               unsigned int iterations,
                               unsigned int initUpdateInterval,
                               unsigned long simDelayFactor)
{
  if (!isRunning()) {
    simMutex.lock();
    meshEngine = engine;
    currentIteration = 0;
    totalIterations = iterations;
    updateInterval = initUpdateInterval;
    delayFactor = simDelayFactor;
    simMutex.unlock();
    start();
  }else{
    //	simMutex.lock();
    //	currentIteration = 0;
    //	totalIterations = iterations;
    //	updateInterval = initUpdateInterval;
    //	delayFactor = simDelayFactor;
    //	restart = true;
    //	simMutex.unlock();

    //	qDebug()<<"Restarting Simulation Thread.";

    //	//condition.wakeOne();
  }
    
}



void MeshSimulationThread::run()
{
  elapsedTime.start();
  simMutex.lock();
  unsigned int threadTotalIterations = totalIterations;
  unsigned int threadUpdateInterval = updateInterval;
  unsigned int threadCurrentIteration = currentIteration;
  unsigned long threadDelayFactor = delayFactor;
  bool abortThread = abort;
  bool pauseThread = pause;
  simMutex.unlock();
  meshEngine->exciteMesh(threadCurrentIteration);
  meshEngine->sampleMesh(threadCurrentIteration);
  while(threadCurrentIteration < threadTotalIterations)
  {
    if(pause){
      simMutex.lock();
      condition.wait(&simMutex);
      pause = false;
      simMutex.unlock();
    }

    if(abort){
      return;
    }

    ++threadCurrentIteration;
    meshEngine->simulateIteration(threadCurrentIteration);

    if(threadCurrentIteration % threadUpdateInterval == 0){
      meshEngine->updateHost();
    }

    simMutex.lock();
    currentIteration = threadCurrentIteration;
    if(threadDelayFactor > 0 && abort != true){
      condition.wait(&simMutex,threadDelayFactor);
    }
    simMutex.unlock();
  }
  meshEngine->updateHost();
  qDebug()<<"Ms Elapsed:"<<elapsedTime.elapsed();
  emit simulationComplete();

}



void MeshSimulationThread::stopSimulation()
{
  if(isRunning()){
    simMutex.lock();
    abort = true;
    simMutex.unlock();
    condition.wakeOne();
    wait();
    abort = false;
  }
}



void MeshSimulationThread::pauseSimulation()
{
  if(isRunning()){
    simMutex.lock();
    pause = true;
    simMutex.unlock();
    condition.wakeOne();
  }
}



void MeshSimulationThread::resumeSimulation()
{
  condition.wakeOne();
}

